@@ -817,8 +817,15 @@ add_highlights_to_geoms = function(geoms, highlight_geoms, highlight_data, annot
             }
 
             # reorder to match the data order:
+
+            # fix for the bug https://github.com/krassowski/complex-upset/issues/122
+            if (!is.factor(highlight_data[[kind]])) {
+                highlight_data[[kind]] = as.factor(highlight_data[[kind]])
+            }
+            data_order = levels(highlight_data[, kind])
+
             params = params[
-                match(unique(highlight_data[, kind]), params$query),
+                match(data_order, params$query),
                 colnames(params) != 'query',
                 drop=FALSE
             ]
@@ -980,14 +987,20 @@ upset = function(
 
   matrix_intersect_queries = intersect_queries(queries_for(queries, 'intersections_matrix'), data)
   matrix_group_by_queries = group_by_queries(queries_for(queries, 'intersections_matrix'), data$sanitized_labels)
+  matrix_set_queries = set_queries(queries_for(queries, 'intersections_matrix'), data$sanitized_labels)
 
   intersection_query_matrix = get_highlights_data(data$matrix_frame, 'intersection', matrix_intersect_queries)
   group_query_matrix = get_highlights_data(data$matrix_frame, 'group_by_group', matrix_group_by_queries)
+  set_query_matrix = get_highlights_data(data$matrix_frame, 'group', matrix_set_queries)
+
 
   query_matrix = merge_rows(
-      intersection_query_matrix,
-      group_query_matrix
-  )
+      merge_rows(
+          intersection_query_matrix,
+          group_query_matrix
+      ),
+      set_query_matrix
+    )
 
   query_matrix = query_matrix[query_matrix$value == TRUE, ]
 
@@ -1019,13 +1032,6 @@ upset = function(
         size=dot_size,
         na.rm=TRUE
     )),
-    # the highlighted dot
-    highlight_layer(
-        intersections_matrix$geom,
-        geom_point,
-        query_matrix,
-        args=list(size=dot_size)
-    ),
     # interconnectors on the dots
     list(intersections_matrix$segment * geom_segment(aes(
           x=intersection,
@@ -1047,6 +1053,13 @@ upset = function(
              ),
              na.rm=TRUE
         )
+    ),
+    # the highlighted dot
+    highlight_layer(
+        intersections_matrix$geom,
+        geom_point,
+        query_matrix,
+        args=list(size=dot_size)
     )
   )
 
@@ -1210,14 +1223,20 @@ upset = function(
   }
 
   if (show_overall_sizes) {
-      overall_sizes_queries = set_queries(queries_for(queries, 'overall_sizes'), data$sanitized_labels)
-      overall_sizes_highlights_data = get_highlights_data(data$presence, 'group', overall_sizes_queries)
+      set_sizes_data = data$presence[data$presence$group %in% data$plot_sets_subset, ]
 
+      if (set_sizes$filter_intersections) {
+          set_sizes_data = set_sizes_data[set_sizes_data$intersection %in% data$plot_intersections_subset, ]
+      }
+
+      overall_sizes_queries = set_queries(queries_for(queries, 'overall_sizes'), data$sanitized_labels)
+      overall_sizes_highlights_data = get_highlights_data(set_sizes_data, 'group', overall_sizes_queries)
       if (nrow(overall_sizes_queries) != 0) {
         highlight_geom = set_sizes$highlight_geom
         if (!inherits(highlight_geom, 'list')) {
             highlight_geom = list(highlight_geom)
         }
+        overall_sizes_highlights_data$group = factor(overall_sizes_highlights_data$group)
         geom = add_highlights_to_geoms(
             set_sizes$geom,
             highlight_geom,
@@ -1235,12 +1254,6 @@ upset = function(
           default_scale = scale_y_reverse()
       }
 
-      set_sizes_data = data$presence[data$presence$group %in% data$plot_sets_subset, ]
-
-      if (set_sizes$filter_intersections) {
-          set_sizes_data = set_sizes_data[set_sizes_data$intersection %in% data$plot_intersections_subset, ]
-      }
-
       set_sizes$layers = c(
           matrix_background_stripes(data, stripes, 'vertical'),
           geom,
